CAMERA = "Camera"
CAMERA_DATA_PATH = "camera_data_path"
CAMERA_1_NAME = "camera-40547499"
CAMERA_2_NAME = "camera-40579627"
CAMERA_3_NAME = "camera-40580655"

TESSERACT = "Tesseract"
IMAGE_DEMO = "image_demo"
OUTPUT_IMAGE_PATH = "output_image_path"
TMP_PATH = "tmp_path"
BENTOML_URL = "bentoml_url"
DEFAULT_CONF = 0.6
CONF_MINIMUM = "conf_minimum"

DEFAULT_CONFIG_FILE_NAME = "./config/cadivi_config.props"



import os
import json
import hashlib
from typing import Tuple, Dict, List, Any

import cv2
import numpy as np
import requests


# Function to decode base64 frame
def decode_base64_frame(encoded_frame):
    np_data = np.frombuffer(encoded_frame, np.uint8)
    return cv2.imdecode(np_data, cv2.IMREAD_COLOR)


# Function to encode frame to base64
def encode_frame_base64(frame):
    _, buffer = cv2.imencode(".jpg", frame)
    return buffer.tobytes()


def write_frame(img, image_file):
    cv2.imwrite(
        f"{image_file}.png",
        img,
    )


def write_temp_file(obd, mime_type, file_path) -> Tuple[str, str]:
    h = hashlib.sha1()
    h.update(obd)
    os.makedirs(file_path, exist_ok=True)

    if mime_type in ["jpg", "jpeg"]:
        image_name = f"{h.hexdigest()}.jpg"
    elif mime_type == "png":
        image_name = f"{h.hexdigest()}.png"
    else:
        raise ValueError(f"Unsupported mime_type: {mime_type}")

    # file_path = config_util.get_property(
    #     section=ConfigConst.TESSERACT, key=ConfigConst.TMP_PATH
    # )
    file_name = file_path + f"/{image_name}"

    with open(file_name, "wb") as handle:
        handle.write(obd)
    return file_name, image_name


def request_bentoml(
    model_id: str, images: List[str], bentoml_url: str, conf_minimum: float
) -> List[Dict[Any, Any]]:
    payload = {"model_id": model_id}

    files = [
        ("images", ("file", open(f"{image}", "rb"), "application/octet-stream"))
        for i, image in enumerate(images)
    ]

    headers = {"accept": "application/json"}

    response = requests.request(
        "POST", bentoml_url, headers=headers, data=payload, files=files
    )

    if response.status_code != 200:
        raise Exception(f"Failed {response.status_code}: {response.text}")

    data = json.loads(response.text)

    filtered_data = [
        {
            "label": "".join(
                char
                for char, conf in zip(item["labels"], map(float, item["conf"]))
                if conf > conf_minimum
            ),
            "conf": [
                conf for conf in map(float, item["conf"]) if conf > conf_minimum
            ],
        }
        for item in data
    ]
    return filtered_data



import os
import threading

# import time

# import cv2
import numpy as np
import pypylon.pylon as py

from cadivi_analysis.settings import logging, camera_producer

# from cadivi_analysis.utils.image_utils import encode_frame_base64, write_frame
from cadivi_analysis.utils.image_utils import write_frame
from cadivi_analysis.common.config_util import ConfigUtil
from cadivi_analysis.common import config_const as ConfigConst

SCANLINE_WIDTH = 1000
VIRTUAL_FRAME_WIDTH = 1000


class CameraUtils:
    def __init__(self, device, offset_x, offset_y, exporsure_time):
        self.cam = py.InstantCamera(
            py.TlFactory.GetInstance().CreateDevice(device)
        )

        self.offset_x = offset_x
        self.offset_y = offset_y
        self.exporsure_time = exporsure_time
        self.config_camera()
        self.name = self.cam.GetDeviceInfo().GetSerialNumber()

        config_util = ConfigUtil()
        self.output_image_path = config_util.get_property(
            section=ConfigConst.CAMERA, key=ConfigConst.CAMERA_DATA_PATH
        )
        self.output_image_path = self.output_image_path + f"/camera-{self.name}"

        os.makedirs(self.output_image_path, exist_ok=True)

    def delete_output_image_path(self):
        for item in os.listdir(self.output_image_path):
            if item.endswith(".png"):
                os.remove(os.path.join(self.output_image_path, item))

    def config_camera(self):
        self.cam.Open()

        self.cam.Width.SetValue(SCANLINE_WIDTH)
        self.cam.Height.SetValue(1000)
        self.cam.OffsetX.SetValue(self.offset_x)
        self.cam.OffsetY.SetValue(self.offset_y)

        self.cam.PixelFormat = "BGR8"
        self.cam.ExposureTime.SetValue(self.exporsure_time)
        self.cam.Gain.SetValue(0)

        # Frame Rate
        self.cam.BandwidthReserveMode.SetValue("Performance")

        logging.info(
            f"Resulting framerate: {self.cam.ResultingFrameRate.Value}"
        )

    def grab_frame(
        self, save: bool = False, is_count_100: bool = False
    ) -> None:
        # Set camera to continuous acquisition mode
        self.cam.AcquisitionMode.SetValue("Continuous")
        self.cam.StartGrabbing(py.GrabStrategy_LatestImageOnly)

        img = np.ones(
            (self.cam.Height.Value, VIRTUAL_FRAME_WIDTH, 3), dtype=np.uint8
        )
        missing_column = (
            np.ones((self.cam.Height.Value, SCANLINE_WIDTH, 3), dtype=np.uint8)
            * 255
        )
        image_idx = 0
        while self.cam.IsGrabbing():
            if image_idx == 100 and is_count_100:
                image_idx = 0

            for idx in range(VIRTUAL_FRAME_WIDTH // SCANLINE_WIDTH):
                with self.cam.RetrieveResult(
                    2000, py.TimeoutHandling_ThrowException
                ) as result:
                    try:
                        if result.GrabSucceeded():
                            out_array = result.GetArray()
                            img[
                                :,
                                idx * SCANLINE_WIDTH : idx * SCANLINE_WIDTH
                                + SCANLINE_WIDTH,
                            ] = out_array
                        else:
                            img[
                                :,
                                idx * SCANLINE_WIDTH : idx * SCANLINE_WIDTH
                                + SCANLINE_WIDTH,
                            ] = missing_column
                            logging.info(f"Missing column at index {idx}")
                        # time.sleep(0.03)
                    except py.RuntimeException as e:
                        logging.error(e)
                        result.Release()

            # Save the resulting frame as an image
            if save:
                write_frame(
                    img,
                    f"{self.output_image_path}/{image_idx}",
                )

            # logging.info(f"{self.name")
            camera_producer.send(
                f"camera-{self.name}",
                f"{self.output_image_path}/{image_idx}.png",
            )

            # Kafka
            # frame = encode_frame_base64(img)
            # camera_producer.send(
            #     f"camera-{self.name}", frame
            # )

            image_idx += 1

    def start_grabbing(
        self, save: bool = False, is_count_100: bool = False
    ) -> None:
        self.thread = threading.Thread(
            target=self.grab_frame, args=(save, is_count_100)
        )
        # self.thread = threading.Thread(target=self.grab_frame)
        self.thread.start()

    def stop_grabbing(self):
        self.cam.StopGrabbing()
        self.cam.Close()
        self.thread.join()



import cv2
import imutils
import numpy as np
from skimage.segmentation import clear_border


class WordDetector:
    def __init__(self, image) -> None:
        self.image = image

    def get_grayscale(self):
        return cv2.cvtColor(self.image, cv2.COLOR_BGR2GRAY)

    def get_morphological(self, image, operation=cv2.MORPH_BLACKHAT):
        rect_kern = cv2.getStructuringElement(cv2.MORPH_RECT, (242, 10))
        black_hat = cv2.morphologyEx(image, operation, rect_kern)
        return black_hat

    def get_light(
        self,
        image,
        thresh: int = 0,
        operation=cv2.MORPH_CLOSE,
    ):
        square_kern = cv2.getStructuringElement(cv2.MORPH_RECT, (thresh, 1))
        light = cv2.morphologyEx(image, operation, square_kern)
        light = cv2.threshold(
            light, 0, 200, cv2.THRESH_BINARY | cv2.THRESH_OTSU
        )[1]
        return light

    def get_grad(self, image):
        gradX = cv2.Sobel(image, ddepth=cv2.CV_32F, dx=1, dy=0, ksize=-1)
        gradX = np.absolute(gradX)
        (minVal, maxVal) = (np.min(gradX), np.max(gradX))
        gradX = 255 * ((gradX - minVal) / (maxVal - minVal))
        gradX = gradX.astype("uint8")
        return gradX

    def get_thresh(self, image):
        rect_kern = cv2.getStructuringElement(cv2.MORPH_RECT, (35, 20))
        gradX = cv2.GaussianBlur(image, (1, 1), 0)
        gradX = cv2.morphologyEx(gradX, cv2.MORPH_CLOSE, rect_kern)
        thresh = cv2.threshold(
            gradX, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU
        )[1]
        return thresh

    def get_ero_dil(self, image):
        thresh = cv2.erode(image, None, iterations=2)
        thresh = cv2.dilate(thresh, None, iterations=2)
        return thresh

    def get_final(self, image, light):
        thresh = cv2.bitwise_and(image, image, mask=light)
        thresh = cv2.dilate(thresh, None, iterations=2)
        thresh = cv2.erode(thresh, None, iterations=1)
        return thresh

    def get_contours(self, raw, image, keep=5):
        cnts = cv2.findContours(
            image.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE
        )
        cnts = imutils.grab_contours(cnts)
        cnts = sorted(cnts, key=cv2.contourArea, reverse=True)[:keep]
        result = cv2.drawContours(np.copy(raw), cnts, -1, (0, 255, 0), 3)
        return cnts, result

    def locate_cadivi_text(
        self, gray, candidates, clearBorder=False, operation=cv2.MORPH_TOPHAT
    ):
        image_cnt = None
        rois = []
        result_images = []

        for c in candidates:
            (x, y, w, h) = cv2.boundingRect(c)
            ar = w / float(h)
            if ar <= 10:
                image_cnt = c
                result_image = gray[y : y + h, x : x + w]
                roi = self.get_morphological(result_image, operation)

                if clearBorder:
                    roi = clear_border(roi)
                rois.append(roi)
                result_images.append(result_image)
        return result_images, rois, image_cnt



import os

import cv2

from cadivi_analysis.common.config_util import ConfigUtil
from cadivi_analysis.common import config_const as ConfigConst
from cadivi_analysis.utils.image_utils import write_frame, request_bentoml
from cadivi_analysis.word_detector.word_detector import WordDetector


def test_tesseract():
    config_util = ConfigUtil()
    input_image_path = config_util.get_property(
        section=ConfigConst.TESSERACT, key=ConfigConst.IMAGE_DEMO
    )
    bentoml_url = config_util.get_property(
        section=ConfigConst.TESSERACT, key=ConfigConst.BENTOML_URL
    )
    conf_minimum = config_util.get_float(
        section=ConfigConst.TESSERACT,
        key=ConfigConst.CONF_MINIMUM,
        default_val=ConfigConst.DEFAULT_CONF,
    )

    # Read Image
    image = cv2.imread(input_image_path)

    # Processing image to get words
    word_detector = WordDetector(image)
    gray = word_detector.get_grayscale()
    blackhat = word_detector.get_morphological(gray, operation=cv2.MORPH_TOPHAT)
    light = word_detector.get_light(
        blackhat, thresh=140, operation=cv2.MORPH_GRADIENT
    )
    cnts, result = word_detector.get_contours(image, light, keep=2)
    result_images, rois, _ = word_detector.locate_cadivi_text(
        gray, cnts, True, operation=cv2.MORPH_TOPHAT
    )

    # Save Image
    output_image_path = config_util.get_property(
        section=ConfigConst.TESSERACT, key=ConfigConst.OUTPUT_IMAGE_PATH
    )
    os.makedirs(output_image_path, exist_ok=True)

    images = []
    for i, roi in enumerate(result_images):
        write_frame(roi, f"{output_image_path}/output_image_{i}")
        images.append(f"{output_image_path}/output_image_{i}.png")

    # Word Detector by BentoML
    try:
        filtered_data = request_bentoml(
            "trba", images, bentoml_url, conf_minimum
        )
        print(filtered_data)
    except Exception as e:
        print(e)


if __name__ == "__main__":
    test_tesseract()



class Singleton(type):
    _instances = {}

    def __call__(c, *args, **kwargs):
        if c not in c._instances:
            c._instances[c] = super().__call__(*args, **kwargs)

        return c._instances[c]



import os
import logging

from pydantic_settings import BaseSettings
from kafka import KafkaProducer, KafkaConsumer

logging.basicConfig(
    level=logging.INFO,
    format="%(levelname)s: %(asctime)s - %(funcName)s - %(message)s",
)

basedir = os.path.abspath(os.path.dirname(__file__))


class Settings(BaseSettings):
    DEBUG: bool = False
    DEVELOPMENT: bool = False
    TESTING: bool = False
    KAFKA_URL: str = os.getenv("KAFKA_URL", "localhost:9092")


settings = Settings()

producer_config = {
    "bootstrap_servers": settings.KAFKA_URL,
    "value_serializer": lambda v: v.encode("utf-8"),
}

camera_producer = KafkaProducer(**producer_config)
camera99_consumer = KafkaConsumer(
    "camera-40547499", bootstrap_servers=[settings.KAFKA_URL]
)
camera27_consumer = KafkaConsumer(
    "camera-40579627", bootstrap_servers=[settings.KAFKA_URL]
)
camera55_consumer = KafkaConsumer(
    "camera-40580655", bootstrap_servers=[settings.KAFKA_URL]
)



import pypylon.pylon as py

from cadivi_analysis.settings import (
    camera99_consumer,
    camera27_consumer,
    camera55_consumer,
    # logging
)
from cadivi_analysis.utils.camera_utils import CameraUtils

# from cadivi_analysis.utils.image_utils import decode_base64_frame, write_frame


def test_grab_image():
    tlf = py.TlFactory.GetInstance()
    devices = tlf.EnumerateDevices()

    camera_99 = CameraUtils(
        devices[0], offset_x=0, offset_y=342, exporsure_time=500
    )
    camera_27 = CameraUtils(
        devices[1], offset_x=0, offset_y=502, exporsure_time=500
    )
    camera_55 = CameraUtils(
        devices[2], offset_x=0, offset_y=292, exporsure_time=500
    )

    # Start Grabbing
    camera_99.start_grabbing(save=True, is_count_100=True)
    camera_27.start_grabbing(save=True)
    camera_55.start_grabbing(save=True)

    count = 0
    while True:
        if count > 150:
            break

        message_99 = camera99_consumer.poll()
        if not len(message_99.items()) == 0:
            for _, records in message_99.items():
                for rec in records:
                    print(rec.value)

        message_27 = camera27_consumer.poll()
        if not len(message_27.items()) == 0:
            for _, records in message_27.items():
                for rec in records:
                    print(rec.value)

        message_55 = camera55_consumer.poll()
        if not len(message_55.items()) == 0:
            count += 1
            for _, records in message_55.items():
                for rec in records:
                    print(rec.value)

    # Stop Grabbing
    camera_99.stop_grabbing()
    camera_27.stop_grabbing()
    camera_55.stop_grabbing()

    camera_99.delete_output_image_path()


def test_tesseract():
    pass


if __name__ == "__main__":
    test_grab_image()



import os
import configparser

from cadivi_analysis.common.singleton import Singleton
from cadivi_analysis.common import config_const as ConfigConst
from cadivi_analysis.settings import logging


class ConfigUtil(metaclass=Singleton):
    config_file = ConfigConst.DEFAULT_CONFIG_FILE_NAME
    config_parser = configparser.ConfigParser()
    is_loaded = False

    def __init__(self, config_file: str = None) -> None:
        if config_file is not None:
            self.config_file = config_file

        self._load_config()
        logging.info(f"Created instance of ConfigUtil: {self}")

    def _load_config(self):
        if os.path.exists(self.config_file):
            logging.info(f"Loading config: {self.config_file}")
            self.config_parser.read(self.config_file)
            self.is_loaded = True
        else:
            logging.info(
                f"""Can't load {self.config_file}.
                Trying default: {ConfigConst.DEFAULT_CONFIG_FILE_NAME}"""
            )
            self.config_file = ConfigConst.DEFAULT_CONFIG_FILE_NAME
            self.config_parser.read(self.config_file)
            self.is_loaded = True

        logging.debug(f"Config: {self.config_parser.sections()}")

    def _get_config(self, force_reload: bool = False) -> config_parser:
        if not self.is_loaded or force_reload:
            self._load_config()

        return self.config_parser

    def get_property(
        self,
        section: str,
        key: str,
        default_val: str = None,
        force_reload: bool = False,
    ):
        return self._get_config(force_reload).get(
            section, key, fallback=default_val
        )

    def get_float(
        self,
        section: str,
        key: str,
        default_val: float = 0.0,
        force_reload: bool = False,
    ):
        return self._get_config(force_reload).getfloat(
            section, key, fallback=default_val
        )
